@@ -1,10 +1,13 @@
 import functools
+import json
 import logging
 import re
 import traceback
+from typing import Dict, Optional
+import urllib.parse
 
 from apispec import yaml_utils
-from flask import Blueprint, current_app, jsonify, make_response, request
+from flask import Blueprint, current_app, jsonify, make_response, request, Response
 from flask_babel import lazy_gettext as _
 import jsonschema
 from marshmallow import ValidationError
@@ -53,6 +56,7 @@ from ..const import (
     API_URI_RIS_KEY,
     PERMISSION_PREFIX,
 )
+from ..exceptions import FABException, InvalidOrderByColumnFABException
 from ..security.decorators import permission_name, protect
 
 log = logging.getLogger(__name__)
@@ -127,14 +131,25 @@ def rison(schema=None):
                 try:
                     kwargs["rison"] = prison.loads(value)
                 except prison.decoder.ParserException:
-                    return self.response_400(message="Not a valid rison argument")
+                    if current_app.config.get("FAB_API_ALLOW_JSON_QS", True):
+                        # Rison failed try json encoded content
+                        try:
+                            kwargs["rison"] = json.loads(
+                                urllib.parse.parse_qs(f"{API_URI_RIS_KEY}={value}").get(
+                                    API_URI_RIS_KEY
+                                )[0]
+                            )
+                        except Exception:
+                            return self.response_400(
+                                message="Not a valid rison/json argument"
+                            )
+                    else:
+                        return self.response_400(message="Not a valid rison argument")
             if schema:
                 try:
                     jsonschema.validate(instance=kwargs["rison"], schema=schema)
                 except jsonschema.ValidationError as e:
-                    return self.response_400(
-                        message="Not a valid rison schema {}".format(e)
-                    )
+                    return self.response_400(message=f"Not a valid rison schema {e}")
             return f(self, *args, **kwargs)
 
         return functools.update_wrapper(wraps, f)
@@ -300,6 +315,17 @@ class BaseApi(object):
                 }
             },
         },
+        "403": {
+            "description": "Forbidden",
+            "content": {
+                "application/json": {
+                    "schema": {
+                        "type": "object",
+                        "properties": {"message": {"type": "string"}},
+                    }
+                }
+            },
+        },
         "404": {
             "description": "Not found",
             "content": {
@@ -338,6 +364,53 @@ class BaseApi(object):
         Override custom OpenApi responses
     """
 
+    exclude_route_methods = set()
+    """
+        Does not register routes for a set of builtin ModelRestApi functions.
+        example::
+
+            class ContactModelView(ModelRestApi):
+                datamodel = SQLAInterface(Contact)
+                exclude_route_methods = {"info", "get_list", "get"}
+
+
+        The previous examples will only register the `put`, `post` and `delete` routes
+    """
+    include_route_methods = None
+    """
+        If defined will assume a white list setup, where all endpoints are excluded
+        except those define on this attribute
+        example::
+
+            class ContactModelView(ModelRestApi):
+                datamodel = SQLAInterface(Contact)
+                include_route_methods = {"list"}
+
+
+        The previous example will exclude all endpoints except the `list` endpoint
+    """
+    openapi_spec_methods: Dict = {}
+    """
+        Merge OpenAPI spec defined on the method's doc.
+        For example to merge/override `get_list`::
+
+
+            class GreetingApi(BaseApi):
+                resource_name = "greeting"
+                openapi_spec_methods = {
+                    "greeting": {
+                        "get": {
+                           "description": "Override description",
+                        }
+                    }
+                }
+    """
+    openapi_spec_tag: Optional[str] = None
+    """
+        By default all endpoints will be tagged (grouped) to their class name.
+        Use this attribute to override the tag name
+    """
+
     def __init__(self):
         """
             Initialization of base permissions
@@ -371,6 +444,15 @@ class BaseApi(object):
             self.base_permissions = set()
             is_add_base_permissions = True
         for attr_name in dir(self):
+            # If include_route_methods is not None white list
+            if (
+                self.include_route_methods is not None
+                and attr_name not in self.include_route_methods
+            ):
+                continue
+            # Don't create permission for excluded routes
+            if attr_name in self.exclude_route_methods:
+                continue
             if hasattr(getattr(self, attr_name), "_permission_name"):
                 if is_collect_previous:
                     self.previous_method_permission_name[attr_name] = getattr(
@@ -407,6 +489,15 @@ class BaseApi(object):
             attr = getattr(self, attr_name)
             if hasattr(attr, "_urls"):
                 for url, methods in attr._urls:
+                    # If include_route_methods is not None white list
+                    if (
+                        self.include_route_methods is not None
+                        and attr_name not in self.include_route_methods
+                    ):
+                        continue
+                    if attr_name in self.exclude_route_methods:
+                        log.info(f"Not registering api spec for method {attr_name}")
+                        continue
                     operations = dict()
                     path = self.path_helper(path=url, operations=operations)
                     self.operation_helper(
@@ -414,9 +505,10 @@ class BaseApi(object):
                     )
                     api_spec.path(path=path, operations=operations)
                     for operation in operations:
-                        api_spec._paths[path][operation]["tags"] = [
-                            self.__class__.__name__
-                        ]
+                        openapi_spec_tag = (
+                            self.openapi_spec_tag or self.__class__.__name__
+                        )
+                        api_spec._paths[path][operation]["tags"] = [openapi_spec_tag]
         self.add_apispec_components(api_spec)
 
     def add_apispec_components(self, api_spec):
@@ -427,7 +519,11 @@ class BaseApi(object):
                 _v = {
                     "in": "query",
                     "name": API_URI_RIS_KEY,
-                    "schema": {"$ref": "#/components/schemas/{}".format(k)},
+                    "content": {
+                        "application/json": {
+                            "schema": {"$ref": "#/components/schemas/{}".format(k)}
+                        }
+                    },
                 }
                 # Using private because parameter method does not behave correctly
                 api_spec.components._schemas[k] = v
@@ -435,6 +531,14 @@ class BaseApi(object):
 
     def _register_urls(self):
         for attr_name in dir(self):
+            if (
+                self.include_route_methods is not None
+                and attr_name not in self.include_route_methods
+            ):
+                continue
+            if attr_name in self.exclude_route_methods:
+                log.info(f"Not registering route for method {attr_name}")
+                continue
             attr = getattr(self, attr_name)
             if hasattr(attr, "_urls"):
                 for url, methods in attr._urls:
@@ -468,6 +572,11 @@ class BaseApi(object):
         :param list methods: A list of methods registered for this path
         """
         for method in methods:
+            try:
+                # Check if method openapi spec is overridden
+                override_method_spec = self.openapi_spec_methods[func.__name__]
+            except KeyError:
+                override_method_spec = {}
             yaml_doc_string = yaml_utils.load_operations_from_docstring(func.__doc__)
             yaml_doc_string = yaml.safe_load(
                 str(yaml_doc_string).replace(
@@ -476,6 +585,8 @@ class BaseApi(object):
             )
             if yaml_doc_string:
                 operation_spec = yaml_doc_string.get(method.lower(), {})
+                # Merge docs spec and override spec
+                operation_spec.update(override_method_spec.get(method.lower(), {}))
                 if self.get_method_permission(func.__name__):
                     operation_spec["security"] = [{"jwt": []}]
                 operations[method.lower()] = operation_spec
@@ -524,7 +635,7 @@ class BaseApi(object):
             Returns the permission name for a method
         """
         if self.method_permission_name:
-            return self.method_permission_name.get(method_name)
+            return self.method_permission_name.get(method_name, method_name)
         else:
             if hasattr(getattr(self, method_name), "_permission_name"):
                 return getattr(getattr(self, method_name), "_permission_name")
@@ -549,7 +660,7 @@ class BaseApi(object):
         ]
 
     @staticmethod
-    def response(code, **kwargs):
+    def response(code, **kwargs) -> Response:
         """
             Generic HTTP JSON response method
 
@@ -562,7 +673,7 @@ class BaseApi(object):
         resp.headers["Content-Type"] = "application/json; charset=utf-8"
         return resp
 
-    def response_400(self, message=None):
+    def response_400(self, message: str = None) -> Response:
         """
             Helper method for HTTP 400 response
 
@@ -572,7 +683,7 @@ class BaseApi(object):
         message = message or "Arguments are not correct"
         return self.response(400, **{"message": message})
 
-    def response_422(self, message=None):
+    def response_422(self, message: str = None) -> Response:
         """
             Helper method for HTTP 422 response
 
@@ -582,7 +693,7 @@ class BaseApi(object):
         message = message or "Could not process entity"
         return self.response(422, **{"message": message})
 
-    def response_401(self):
+    def response_401(self) -> Response:
         """
             Helper method for HTTP 401 response
 
@@ -591,7 +702,16 @@ class BaseApi(object):
         """
         return self.response(401, **{"message": "Not authorized"})
 
-    def response_404(self):
+    def response_403(self) -> Response:
+        """
+            Helper method for HTTP 403 response
+
+        :param message: Error message (str)
+        :return: HTTP Json response
+        """
+        return self.response(403, **{"message": "Forbidden"})
+
+    def response_404(self) -> Response:
         """
             Helper method for HTTP 404 response
 
@@ -600,7 +720,7 @@ class BaseApi(object):
         """
         return self.response(404, **{"message": "Not found"})
 
-    def response_500(self, message=None):
+    def response_500(self, message: str = None) -> Response:
         """
             Helper method for HTTP 500 response
 
@@ -1096,6 +1216,15 @@ class ModelRestApi(BaseModelApi):
     def merge_show_title(self, response, **kwargs):
         response[API_SHOW_TITLE_RES_KEY] = self.show_title
 
+    def info_headless(self, **kwargs) -> Response:
+        """
+            response for CRUD REST meta data
+        """
+        _response = dict()
+        _args = kwargs.get("rison", {})
+        self.set_response_key_mappings(_response, self.info, _args, **_args)
+        return self.response(200, **_response)
+
     @expose("/_info", methods=["GET"])
     @protect()
     @safe
@@ -1142,12 +1271,41 @@ class ModelRestApi(BaseModelApi):
             500:
               $ref: '#/components/responses/500'
         """
+        return self.info_headless(**kwargs)
+
+    def get_headless(self, pk, **kwargs) -> Response:
+        """
+            Get an item from Model
+
+        :param pk: Item primary key
+        :param kwargs: Query string parameter arguments
+        :return: HTTP Response
+        """
+        item = self.datamodel.get(pk, self._base_filters)
+        if not item:
+            return self.response_404()
+
         _response = dict()
         _args = kwargs.get("rison", {})
-        self.set_response_key_mappings(_response, self.info, _args, **_args)
+        select_cols = _args.get(API_SELECT_COLUMNS_RIS_KEY, [])
+        _pruned_select_cols = [col for col in select_cols if col in self.show_columns]
+        self.set_response_key_mappings(
+            _response,
+            self.get,
+            _args,
+            **{API_SELECT_COLUMNS_RIS_KEY: _pruned_select_cols},
+        )
+        if _pruned_select_cols:
+            _show_model_schema = self.model2schemaconverter.convert(_pruned_select_cols)
+        else:
+            _show_model_schema = self.show_model_schema
+
+        _response["id"] = pk
+        _response[API_RESULT_RES_KEY] = _show_model_schema.dump(item, many=False).data
+        self.pre_get(_response)
         return self.response(200, **_response)
 
-    @expose("/<pk>", methods=["GET"])
+    @expose("/<int:pk>", methods=["GET"])
     @protect()
     @safe
     @permission_name("get")
@@ -1199,28 +1357,55 @@ class ModelRestApi(BaseModelApi):
             500:
               $ref: '#/components/responses/500'
         """
-        item = self.datamodel.get(pk, self._base_filters)
-        if not item:
-            return self.response_404()
+        return self.get_headless(pk, **kwargs)
 
+    def get_list_headless(self, **kwargs) -> Response:
+        """
+            Get list of items from Model
+        """
         _response = dict()
         _args = kwargs.get("rison", {})
+        # handle select columns
         select_cols = _args.get(API_SELECT_COLUMNS_RIS_KEY, [])
-        _pruned_select_cols = [col for col in select_cols if col in self.show_columns]
+        _pruned_select_cols = [col for col in select_cols if col in self.list_columns]
         self.set_response_key_mappings(
             _response,
-            self.get,
+            self.get_list,
             _args,
             **{API_SELECT_COLUMNS_RIS_KEY: _pruned_select_cols},
         )
-        if _pruned_select_cols:
-            _show_model_schema = self.model2schemaconverter.convert(_pruned_select_cols)
-        else:
-            _show_model_schema = self.show_model_schema
 
-        _response["id"] = pk
-        _response[API_RESULT_RES_KEY] = _show_model_schema.dump(item, many=False).data
-        self.pre_get(_response)
+        if _pruned_select_cols:
+            _list_model_schema = self.model2schemaconverter.convert(_pruned_select_cols)
+        else:
+            _list_model_schema = self.list_model_schema
+        # handle filters
+        try:
+            joined_filters = self._handle_filters_args(_args)
+        except FABException as e:
+            return self.response_400(message=str(e))
+        # handle base order
+        try:
+            order_column, order_direction = self._handle_order_args(_args)
+        except InvalidOrderByColumnFABException as e:
+            return self.response_400(message=str(e))
+        # handle pagination
+        page_index, page_size = self._handle_page_args(_args)
+        # Make the query
+        query_select_columns = _pruned_select_cols or self.list_columns
+        count, lst = self.datamodel.query(
+            joined_filters,
+            order_column,
+            order_direction,
+            page=page_index,
+            page_size=page_size,
+            select_columns=query_select_columns,
+        )
+        pks = self.datamodel.get_keys(lst)
+        _response[API_RESULT_RES_KEY] = _list_model_schema.dump(lst, many=True).data
+        _response["ids"] = pks
+        _response["count"] = count
+        self.pre_get_list(_response)
         return self.response(200, **_response)
 
     @expose("/", methods=["GET"])
@@ -1261,6 +1446,8 @@ class ModelRestApi(BaseModelApi):
                         type: array
                         items:
                           type: string
+                      count:
+                        type: number
                       order_columns:
                         type: array
                         items:
@@ -1278,44 +1465,37 @@ class ModelRestApi(BaseModelApi):
             500:
               $ref: '#/components/responses/500'
         """
-        _response = dict()
-        _args = kwargs.get("rison", {})
-        # handle select columns
-        select_cols = _args.get(API_SELECT_COLUMNS_RIS_KEY, [])
-        _pruned_select_cols = [col for col in select_cols if col in self.list_columns]
-        self.set_response_key_mappings(
-            _response,
-            self.get_list,
-            _args,
-            **{API_SELECT_COLUMNS_RIS_KEY: _pruned_select_cols},
-        )
+        return self.get_list_headless(**kwargs)
 
-        if _pruned_select_cols:
-            _list_model_schema = self.model2schemaconverter.convert(_pruned_select_cols)
-        else:
-            _list_model_schema = self.list_model_schema
-        # handle filters
-        joined_filters = self._handle_filters_args(_args)
-        # handle base order
-        order_column, order_direction = self._handle_order_args(_args)
-        # handle pagination
-        page_index, page_size = self._handle_page_args(_args)
-        # Make the query
-        query_select_columns = _pruned_select_cols or self.list_columns
-        count, lst = self.datamodel.query(
-            joined_filters,
-            order_column,
-            order_direction,
-            page=page_index,
-            page_size=page_size,
-            select_columns=query_select_columns,
-        )
-        pks = self.datamodel.get_keys(lst)
-        _response[API_RESULT_RES_KEY] = _list_model_schema.dump(lst, many=True).data
-        _response["ids"] = pks
-        _response["count"] = count
-        self.pre_get_list(_response)
-        return self.response(200, **_response)
+    def post_headless(self) -> Response:
+        """
+            POST/Add item to Model
+        :return:
+        """
+        if not request.is_json:
+            return self.response_400(message="Request is not JSON")
+        try:
+            item = self.add_model_schema.load(request.json)
+        except ValidationError as err:
+            return self.response_422(message=err.messages)
+        # This validates custom Schema with custom validations
+        if isinstance(item.data, dict):
+            return self.response_422(message=item.errors)
+        self.pre_add(item.data)
+        try:
+            self.datamodel.add(item.data, raise_exception=True)
+            self.post_add(item.data)
+            return self.response(
+                201,
+                **{
+                    API_RESULT_RES_KEY: self.add_model_schema.dump(
+                        item.data, many=False
+                    ).data,
+                    "id": self.datamodel.get_pk_value(item.data),
+                },
+            )
+        except IntegrityError as e:
+            return self.response_422(message=str(e.orig))
 
     @expose("/", methods=["POST"])
     @protect()
@@ -1353,26 +1533,35 @@ class ModelRestApi(BaseModelApi):
             500:
               $ref: '#/components/responses/500'
         """
+        return self.post_headless()
+
+    def put_headless(self, pk) -> Response:
+        """
+            PUT/Edit item to Model
+        """
+        item = self.datamodel.get(pk, self._base_filters)
         if not request.is_json:
-            return self.response_400(message="Request is not JSON")
+            return self.response(400, **{"message": "Request is not JSON"})
+        if not item:
+            return self.response_404()
         try:
-            item = self.add_model_schema.load(request.json)
+            data = self._merge_update_item(item, request.json)
+            item = self.edit_model_schema.load(data, instance=item)
         except ValidationError as err:
             return self.response_422(message=err.messages)
         # This validates custom Schema with custom validations
         if isinstance(item.data, dict):
             return self.response_422(message=item.errors)
-        self.pre_add(item.data)
+        self.pre_update(item.data)
         try:
-            self.datamodel.add(item.data, raise_exception=True)
-            self.post_add(item.data)
+            self.datamodel.edit(item.data, raise_exception=True)
+            self.post_update(item)
             return self.response(
-                201,
+                200,
                 **{
-                    API_RESULT_RES_KEY: self.add_model_schema.dump(
+                    API_RESULT_RES_KEY: self.edit_model_schema.dump(
                         item.data, many=False
-                    ).data,
-                    "id": self.datamodel.get_pk_value(item.data),
+                    ).data
                 },
             )
         except IntegrityError as e:
@@ -1419,31 +1608,20 @@ class ModelRestApi(BaseModelApi):
             500:
               $ref: '#/components/responses/500'
         """
+        return self.put_headless(pk)
+
+    def delete_headless(self, pk) -> Response:
+        """
+            Delete item from Model
+        """
         item = self.datamodel.get(pk, self._base_filters)
-        if not request.is_json:
-            return self.response(400, **{"message": "Request is not JSON"})
         if not item:
             return self.response_404()
+        self.pre_delete(item)
         try:
-            data = self._merge_update_item(item, request.json)
-            item = self.edit_model_schema.load(data, instance=item)
-        except ValidationError as err:
-            return self.response_422(message=err.messages)
-        # This validates custom Schema with custom validations
-        if isinstance(item.data, dict):
-            return self.response_422(message=item.errors)
-        self.pre_update(item.data)
-        try:
-            self.datamodel.edit(item.data, raise_exception=True)
-            self.post_update(item)
-            return self.response(
-                200,
-                **{
-                    API_RESULT_RES_KEY: self.edit_model_schema.dump(
-                        item.data, many=False
-                    ).data
-                },
-            )
+            self.datamodel.delete(item, raise_exception=True)
+            self.post_delete(item)
+            return self.response(200, message="OK")
         except IntegrityError as e:
             return self.response_422(message=str(e.orig))
 
@@ -1477,16 +1655,7 @@ class ModelRestApi(BaseModelApi):
             500:
               $ref: '#/components/responses/500'
         """
-        item = self.datamodel.get(pk, self._base_filters)
-        if not item:
-            return self.response_404()
-        self.pre_delete(item)
-        try:
-            self.datamodel.delete(item, raise_exception=True)
-            self.post_delete(item)
-            return self.response(200, message="OK")
-        except IntegrityError as e:
-            return self.response_422(message=str(e.orig))
+        return self.delete_headless(pk)
 
     """
     ------------------------------------------------
@@ -1534,9 +1703,13 @@ class ModelRestApi(BaseModelApi):
         order_column = rison_args.get(API_ORDER_COLUMN_RIS_KEY, "")
         order_direction = rison_args.get(API_ORDER_DIRECTION_RIS_KEY, "")
         if not order_column and self.base_order:
-            order_column, order_direction = self.base_order
-        if order_column not in self.order_columns:
+            return self.base_order
+        if not order_column:
             return "", ""
+        elif order_column not in self.order_columns:
+            raise InvalidOrderByColumnFABException(
+                f"Invalid order by column: {order_column}"
+            )
         return order_column, order_direction
 
     def _handle_filters_args(self, rison_args):
@@ -1578,7 +1751,8 @@ class ModelRestApi(BaseModelApi):
             ret["validate"] = [str(field.validate)]
         ret["type"] = field.__class__.__name__
         ret["required"] = field.required
-        ret["unique"] = field.unique
+        # When using custom marshmallow schemas fields don't have unique property
+        ret["unique"] = getattr(field, "unique", False)
         return ret
 
     def _get_fields_info(self, cols, model_schema, filter_rel_fields, **kwargs):
