@@ -14,10 +14,10 @@ from .const import (
     LOGMSG_ERR_FAB_ADDON_PROCESS,
     LOGMSG_INF_FAB_ADD_VIEW,
     LOGMSG_INF_FAB_ADDON_ADDED,
-    LOGMSG_WAR_FAB_VIEW_EXISTS
+    LOGMSG_WAR_FAB_VIEW_EXISTS,
 )
 from .filters import TemplateFilters
-from .menu import Menu
+from .menu import Menu, MenuApiManager
 from .views import IndexView, UtilView
 
 log = logging.getLogger(__name__)
@@ -109,7 +109,7 @@ class AppBuilder(object):
         static_folder="static/appbuilder",
         static_url_path="/appbuilder",
         security_manager_class=None,
-        update_perms=True
+        update_perms=True,
     ):
         """
             AppBuilder constructor
@@ -167,42 +167,34 @@ class AppBuilder(object):
 
         self.app = app
 
-        self.base_template = app.config.get(
-            "FAB_BASE_TEMPLATE",
-            self.base_template,
-        )
-        self.static_folder = app.config.get(
-            "FAB_STATIC_FOLDER",
-            self.static_folder,
-        )
+        self.base_template = app.config.get("FAB_BASE_TEMPLATE", self.base_template)
+        self.static_folder = app.config.get("FAB_STATIC_FOLDER", self.static_folder)
         self.static_url_path = app.config.get(
-            "FAB_STATIC_URL_PATH",
-            self.static_url_path,
+            "FAB_STATIC_URL_PATH", self.static_url_path
         )
-        _index_view = app.config.get('FAB_INDEX_VIEW', None)
+        _index_view = app.config.get("FAB_INDEX_VIEW", None)
         if _index_view is not None:
-            self.indexview = dynamic_class_import(
-                _index_view
-            )
+            self.indexview = dynamic_class_import(_index_view)
         else:
             self.indexview = self.indexview or IndexView
-        _menu = app.config.get('FAB_MENU', None)
+        _menu = app.config.get("FAB_MENU", None)
         if _menu is not None:
-            self.menu = dynamic_class_import(
-                _menu
-            )
+            self.menu = dynamic_class_import(_menu)
         else:
             self.menu = self.menu or Menu()
 
         if self.update_perms:  # default is True, if False takes precedence from config
-            self.update_perms = app.config.get('FAB_UPDATE_PERMS', True)
-        _security_manager_class_name = app.config.get('FAB_SECURITY_MANAGER_CLASS', None)
+            self.update_perms = app.config.get("FAB_UPDATE_PERMS", True)
+        _security_manager_class_name = app.config.get(
+            "FAB_SECURITY_MANAGER_CLASS", None
+        )
         if _security_manager_class_name is not None:
             self.security_manager_class = dynamic_class_import(
                 _security_manager_class_name
             )
         if self.security_manager_class is None:
             from flask_appbuilder.security.sqla.manager import SecurityManager
+
             self.security_manager_class = SecurityManager
 
         self._addon_managers = app.config["ADDON_MANAGERS"]
@@ -210,6 +202,7 @@ class AppBuilder(object):
         self.sm = self.security_manager_class(self)
         self.bm = BabelManager(self)
         self.openapi_manager = OpenApiManager(self)
+        self.menuapi_manager = MenuApiManager(self)
         self._add_global_static()
         self._add_global_filters()
         app.before_request(self.sm.before_request)
@@ -322,6 +315,7 @@ class AppBuilder(object):
         self.bm.register_views()
         self.sm.register_views()
         self.openapi_manager.register_views()
+        self.menuapi_manager.register_views()
 
     def _add_addon_views(self):
         """
@@ -633,7 +627,7 @@ class AppBuilder(object):
             for inner_class in view.get_uninit_inner_views():
                 for v in self.baseviews:
                     if (
-                        isinstance(v, inner_class) and
-                            v not in view.get_init_inner_views()
+                        isinstance(v, inner_class)
+                        and v not in view.get_init_inner_views()
                     ):
                         view.get_init_inner_views().append(v)
