@@ -1,9 +1,18 @@
 import datetime
-import logging
 from functools import reduce
-from flask_babelpkg import lazy_gettext
+import logging
+
+from flask_babel import lazy_gettext
+
 from .filters import Filters
 
+try:
+    import enum
+
+    _has_enum = True
+except ImportError:
+    _has_enum = False
+
 log = logging.getLogger(__name__)
 
 
@@ -12,19 +21,26 @@ class BaseInterface(object):
         Base class for all data model interfaces.
         Sub class it to implement your own interface for some data engine.
     """
+
     obj = None
 
     filter_converter_class = None
     """ when sub classing override with your own custom filter converter """
 
     """ Messages to display on CRUD Events """
-    add_row_message = lazy_gettext('Added Row')
-    edit_row_message = lazy_gettext('Changed Row')
-    delete_row_message = lazy_gettext('Deleted Row')
-    delete_integrity_error_message = lazy_gettext('Associated data exists, please delete them first')
-    add_integrity_error_message = lazy_gettext('Integrity error, probably unique constraint')
-    edit_integrity_error_message = lazy_gettext('Integrity error, probably unique constraint')
-    general_error_message = lazy_gettext('General Error')
+    add_row_message = lazy_gettext("Added Row")
+    edit_row_message = lazy_gettext("Changed Row")
+    delete_row_message = lazy_gettext("Deleted Row")
+    delete_integrity_error_message = lazy_gettext(
+        "Associated data exists, please delete them first"
+    )
+    add_integrity_error_message = lazy_gettext(
+        "Integrity error, probably unique constraint"
+    )
+    edit_integrity_error_message = lazy_gettext(
+        "Integrity error, probably unique constraint"
+    )
+    general_error_message = lazy_gettext("General Error")
 
     """ Tuple with message and text with severity type ex: ("Added Row", "info") """
     message = ()
@@ -32,16 +48,40 @@ class BaseInterface(object):
     def __init__(self, obj):
         self.obj = obj
 
-    def _get_attr_value(self, item, col):
+    def _get_attr(self, col_name):
+        if not hasattr(self.obj, col_name):
+            # it's an inner obj attr
+            try:
+                _obj = self.obj
+                for i in col_name.split("."):
+                    try:
+                        _obj = self.get_related_model(i)
+                    except Exception:
+                        _obj = getattr(_obj, i)
+                return _obj
+            except Exception:
+                return None
+        return getattr(self.obj, col_name)
+
+    @staticmethod
+    def _get_attr_value(item, col):
         if not hasattr(item, col):
             # it's an inner obj attr
-            return reduce(getattr, col.split('.'), item)
-        if hasattr(getattr(item, col), '__call__'):
+            try:
+                return reduce(getattr, col.split("."), item)
+            except Exception:
+                return ""
+        if hasattr(getattr(item, col), "__call__"):
             # its a function
             return getattr(item, col)()
         else:
-            # its attribute
-            return getattr(item, col)
+            # its an attribute
+            value = getattr(item, col)
+            # if value is an Enum instance than list and show widgets should display
+            # its .value rather than its .name:
+            if _has_enum and isinstance(value, enum.Enum):
+                return value.value
+            return value
 
     def get_filters(self, search_columns=None):
         search_columns = search_columns or []
@@ -91,7 +131,9 @@ class BaseInterface(object):
         result = []
         for item in self.get_values(lst, list_columns):
             for key, value in list(item.items()):
-                if isinstance(value, datetime.datetime) or isinstance(value, datetime.date):
+                if isinstance(value, datetime.datetime) or isinstance(
+                    value, datetime.date
+                ):
                     value = value.isoformat()
                     item[key] = value
                 if isinstance(value, list):
@@ -103,6 +145,7 @@ class BaseInterface(object):
         Returns the models class name
         useful for auto title on views
     """
+
     @property
     def model_name(self):
         return self.obj.__class__.__name__
@@ -110,8 +153,15 @@ class BaseInterface(object):
     """
         Next methods must be overridden
     """
-    def query(self, filters=None, order_column='', order_direction='',
-              page=None, page_size=None):
+
+    def query(
+        self,
+        filters=None,
+        order_column="",
+        order_direction="",
+        page=None,
+        page_size=None,
+    ):
         pass
 
     def is_image(self, col_name):
@@ -132,6 +182,9 @@ class BaseInterface(object):
     def is_text(self, col_name):
         return False
 
+    def is_binary(self, col_name):
+        return False
+
     def is_integer(self, col_name):
         return False
 
@@ -150,6 +203,9 @@ class BaseInterface(object):
     def is_datetime(self, col_name):
         return False
 
+    def is_enum(self, col_name):
+        return False
+
     def is_relation(self, prop):
         return False
 
@@ -177,6 +233,9 @@ class BaseInterface(object):
     def is_pk(self, col_name):
         return False
 
+    def is_pk_composite(self):
+        raise False
+
     def is_fk(self, col_name):
         return False
 
@@ -218,20 +277,28 @@ class BaseInterface(object):
             return a list of pk values from object list
         """
         pk_name = self.get_pk_name()
-        return [getattr(item, pk_name) for item in lst]
+        if self.is_pk_composite():
+            return [[getattr(item, pk) for pk in pk_name] for item in lst]
+        else:
+            return [getattr(item, pk_name) for item in lst]
 
-    def get_pk_name(self, item):
+    def get_pk_name(self):
         """
             Returns the primary key name
         """
         raise NotImplementedError
 
     def get_pk_value(self, item):
-        return getattr(item, self.get_pk_name())
+        pk_name = self.get_pk_name()
+        if self.is_pk_composite():
+            return [getattr(item, pk) for pk in pk_name]
+        else:
+            return getattr(item, pk_name)
 
-    def get(self, pk):
+    def get(self, pk, filter=None):
         """
-            return the record from key
+            return the record from key, you can optionally pass filters
+            if pk exits on the db but filters exclude it it will return none.
         """
         pass
 
@@ -280,6 +347,3 @@ class BaseInterface(object):
 
     def get_relation_fk(self, prop):
         pass
-
-
-
